<?php
/**
 * AdminInit
 * Version    : 0.1
 * Author     : Green Sheep
 * Created    : November 2, 2017
 * Modified   : November 12, 2017
 * License    : GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */
namespace plugins\wpv;

class AdminInit
{
    /**
     * 管理画面に追加
     */
    public function __construct()
    {
        add_action('admin_menu', array($this, 'addAdminMenuPanorama'));
        add_action('admin_menu', array($this, 'wpPanoramaViewMenu'));

        // 「メディアを追加」から「投稿に挿入」したときにパノラマ画像用のショートコードタグを生成
        add_filter('get_image_tag', array($this, 'setPanoramaImageTag'), 10, 6);
        // 「メディアを追加」の添付ファイルの表示設定のサイズオプションに「パノラマ」を追加
        add_action('post-upload-ui', array($this, 'startMediaScriptBuffer'));
        add_action('print_media_templates', array($this, 'getMediaScriptBuffer'));
    }

    /**
     * パノラマ画像用のカスタムフィールドを追加
     */
    public function addAdminMenuPanorama()
    {
        new AdminMenuPanorama();
    }

    /**
     * 管理画面のメニューに地図設定を追加する
     *
     * パノラマ画像設定
     */
    public function wpPanoramaViewMenu()
    {
        $icon_path = plugins_url() . '/' . basename(WPV_PLUGIN_PATH) . '/images/admin-menu-icon.png';

        add_menu_page(
            'パノラマ設定',
            'パノラマ設定',
            'manage_options',
            'wp-panorama-view-setting',
            array($this, 'addAdminMenuInitSetting'),
            $icon_path
        );
    }

    /**
     * 初期設定用のページを呼び出す
     */
    public function addAdminMenuInitSetting()
    {
        new AdminMenuInitSetting();
    }

    /**
     * 「メディアを追加」から「投稿に挿入」したときにパノラマ画像用のショートコードタグを生成
     *
     * @param string $html htmlタグ
     * @param string $id 画像ID
     * @param string $alt alt属性
     * @param string $title タイトル
     * @param string $align 配置
     * @param string $size サイズ
     *
     * @return string htmlタグ
     */
    public function setPanoramaImageTag($html, $id, $alt, $title, $align, $size)
    {
        // サイズにパノラマを設定していない場合は何もしない
        if ($size !== 'panorama') {
            return $html;
        }

        $shortcode = '';

        // 画像のソースを取り出す
        $pattern = '/<img.*?src=\"(.*?)\".*>/i';
        if (preg_match($pattern, $html, $src)) {
            $shortcode = "[wp-panorama-view src={$src[1]}]";
            return $shortcode;
        }

        // 画像のソースを取得できなかった場合は空文字を返す
        return '';
    }

    /**
     * メディアを挿入画面の出力をバッファリングする
     */
    public function startMediaScriptBuffer() {
        ob_start();
    }

    /**
     * メディアを挿入画面の添付ファイルの表示設定のサイズオプションに「パノラマ」を追加
     * @return [type] [description]
     */
    public function getMediaScriptBuffer() {
        $scripts = ob_get_clean();
        $scripts = preg_replace(
            '#<option value="full"(.*?)</option>#s',
            '<option value="full"$1</option><option value="panorama">パノラマ</option>',
            $scripts
        );
        echo $scripts;
    }
}
