<?php
/**
 * AdminMenuInitSetting
 * Version    : 0.1
 * Author     : Green Sheep
 * Created    : November 12, 2017
 * Modified   :
 * License    : GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */
namespace plugins\wpv;

class AdminMenuInitSetting extends AbstractAdminMenu
{
    protected function renderPage()
    {
        // 登録ボタン等が押された場合に然るべき処理を行う
        $this->executeSubmit();

        // 設定値の取得
        $setting_value = Cmn::getSettingValue();
        ?>
        <div class="wrap">
            <h1>WP Panorama View</h1>
            <p>パノラマ画像表示に関する設定を行います。</p>
            <form method="post">
                <?php wp_nonce_field('wp_panorama_view_action', 'wp_panorama_view_field');
                if (!empty($_POST)) {
                    if (isset($_POST['error'])) {
                        $class_name = 'error';
                        $str = '設定の保存に失敗しました';
                    } else {
                        $class_name = 'updated';
                        $str = '設定を保存しました';
                    } ?>
                    <div class="<?php echo $class_name; ?> notice is-dismissible">
                        <p><strong><?php echo $str; ?></strong></p>
                        <button class="notice-dismiss" type="button">
                            <span class="screen-reader-text">この通知を非表示にする</span>
                        </button>
                    </div>
                <?php } ?>
                <hr>
                <h2>パノラマ画像表示設定</h2>
                <table class="form-table">
                    <tr>
                        <th scope="row">オートローテーション</th>
                        <td>
                            <label><input type="radio" name="autolotation" value="false" <?php if ($setting_value['autolotation'] === 'false') echo "checked"; ?>>しない</label><br>
                            <label><input type="radio" name="autolotation" value="true" <?php if ($setting_value['autolotation'] === 'true') echo "checked"; ?>>する</label>
                            <p class="description">パノラマ画像の初期表示または切り替え時の表示の際に自動で回転するか選択します。</p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">オートローテーション速度</th>
                        <td>
                            <?php if (isset($_POST['error']['autolotation_speed']) && $_POST['error']['autolotation_speed'] === 1) {
                                echo '<p class="error-message">オートローテーション速度は-2～2で登録してください</p>';
                            } ?>
                            <input type="number" id="autolotation_speed" name="autolotation_speed" value="<?php echo $setting_value['autolotation_speed']; ?>" step="0.1" min="-2" max="2"> （初期値0.6 : -2～2）
                            <p class="description">自動回転時の速度を選択します。マイナスの数値を入力すると反対の回転を行います。</p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">手動ローテーション速度</th>
                        <td>
                            <?php if (isset($_POST['error']['lotation_speed']) && $_POST['error']['lotation_speed'] === 1) {
                                echo '<p class="error-message">手動ローテーション速度は-0.8～0.8で登録してください</p>';
                            } ?>
                            <input type="number" id="lotation_speed" name="lotation_speed" value="<?php echo $setting_value['lotation_speed']; ?>" step="0.1" min="-0.8" max="0.8"> （初期値-0.1 : -0.8～0.8）
                            <p class="description">手動回転時の速度を選択します。マイナスの数値を入力すると反対の回転を行います。</p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">ズームの範囲（最小値）</th>
                        <td>
                            <?php if (isset($_POST['error']['min_distance']) && $_POST['error']['min_distance'] === 1) {
                                echo '<p class="error-message">ズームの範囲は0から10で登録してください。</p>';
                            } ?>
                            <input type="number" id="min_distance" name="min_distance" value="<?php echo $setting_value['min_distance']; ?>" step="0.1" min="0" max="10"> （初期値0 : 0～10）
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">ズームの範囲（最大値）</th>
                        <td>
                            <?php if (isset($_POST['error']['max_distance']) && $_POST['error']['max_distance'] === 1) {
                                echo '<p class="error-message">ズームの範囲は0から10で登録してください。</p>';
                            } ?>
                            <input type="number" id="max_distance" name="max_distance" value="<?php echo $setting_value['max_distance']; ?>" step="0.1" min="0" max="10"> （初期値5 : 0～10）
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">ズーム速度</th>
                        <td>
                            <?php if (isset($_POST['error']['zoom_speed']) && $_POST['error']['zoom_speed'] === 1) {
                                echo '<p class="error-message">ズーム速度は0～30で登録してください</p>';
                            } ?>
                            <input type="number" id="zoom_speed" name="zoom_speed" value="<?php echo $setting_value['zoom_speed']; ?>" min="0" max="30"> （初期値10 : 0～30）
                            <p class="description">ズーム時の速度を調節します。</p>
                        </td>
                    </tr>
                </table>
                <hr>
                <h2>WP Panorama Viewに関するデータの削除</h2>
                <p>プラグインの無効化時にWP Panorama Viewプラグインが作成したオプションデータを削除するか選択できます。</p>
                <table class="form-table">
                    <tr>
                        <th scope="row">関連データの削除</th>
                        <td>
                            <label><input type="radio" name="delete_flg" value="0" <?php if ($setting_value['delete_flg'] === 0) echo "checked"; ?>>削除しない</label><br>
                            <label><input type="radio" name="delete_flg" value="1" <?php if ($setting_value['delete_flg'] === 1) echo "checked"; ?>>削除する</label>
                        </td>
                    </tr>
                </table>
                <input type="submit" name="regist_wpv_detail" id="regist_wpv_detail" class="button-primary" value="変更を保存">
            </form>
        </div>
        <?php
    }

    /**
     * フォームの送信が行われた際に然るべき処理を行う
     *
     * @return boolean
     */
    private function executeSubmit()
    {
        // nonce check
        if (!(isset($_POST['wp_panorama_view_field']) && wp_verify_nonce($_POST['wp_panorama_view_field'], 'wp_panorama_view_action') === 1)) {
            return false;
        }

        if (isset($_POST['regist_wpv_detail'])) {
            // オートローテーション
            if ($_POST['autolotation'] === 'false') {
                DAO::updateOption('autolotation', $_POST['autolotation']);
            } else {
                DAO::updateOption('autolotation', 'true');
            }

            // オートローテーション速度
            $autolotation_speed = $_POST['autolotation_speed'] !== '' ? (float)$_POST['autolotation_speed'] : Cmn::DEFAULT_AUTOLOTATION_SPEED;
            if ($autolotation_speed >= -2 && $autolotation_speed <= 2) {
                DAO::updateOption('autolotation_speed', $autolotation_speed);
            } else {
                $_POST['error']['autolotation_speed'] = 1;
            }

            // 手動ローテーション速度
            $lotation_speed = $_POST['lotation_speed'] !== '' ? (float)$_POST['lotation_speed'] : Cmn::DEFAULT_LOTATION_SPEED;
            if ($lotation_speed >= -0.8 && $lotation_speed <= 0.8) {
                DAO::updateOption('lotation_speed', $lotation_speed);
            } else {
                $_POST['error']['lotation_speed'] = 1;
            }

            $min_distance = $_POST['min_distance'] !== '' ? (float)$_POST['min_distance'] : Cmn::DEFAULT_MIN_DISTANCE;
            $max_distance = $_POST['max_distance'] !== '' ? (float)$_POST['max_distance'] : Cmn::DEFAULT_MAX_DISTANCE;

            // 最大値と最小値が反対の場合は入れ替える
            if ($min_distance > $max_distance) {
                $tmp            = $min_distance;
                $min_distance    = $max_distance;
                $max_distance    = $tmp;
            }

            // ズームレベルの最小値の更新
            if ($min_distance >= 0 && $min_distance <= 10) {
                DAO::updateOption('min_distance', $min_distance);
            } else {
                $_POST['error']['min_distance'] = 1;
            }

            // ズームレベルの最大値の更新
            if ($max_distance >= 0 && $max_distance <= 10) {
                DAO::updateOption('max_distance', $max_distance);
            } else {
                $_POST['error']['max_distance'] = 1;
            }

            // ズーム速度
            $zoom_speed = $_POST['zoom_speed'] !== '' ? (int)$_POST['zoom_speed'] : Cmn::DEFAULT_ZOOM_SPEED;
            if ($zoom_speed >= 0 && $zoom_speed <= 30) {
                DAO::updateOption('zoom_speed', $zoom_speed);
            } else {
                $_POST['error']['zoom_speed'] = 1;
            }

            // プラグイン無効化時の関連データの削除
            if ($_POST['delete_flg'] === '0' || $_POST['delete_flg'] === '1') {
                DAO::updateOption('delete_flg', $_POST['delete_flg']);
            }
        }
    }
}
