<?php
/**
 * Cmn
 * description: 共通の処理を行うクラス
 * Version    : 0.1
 * Author     : Green Sheep
 * Created    : November 2, 2017
 * Modified   :
 * License    : GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */
namespace plugins\wpv;

class Cmn
{
    /** プラグインプレフィックス */
    const PREFIX = 'wpv_';

    const UNIQUE_KEY = 'r3ZKNi_';

    // オートローテーション速度のデフォルト値
    const DEFAULT_AUTOLOTATION_SPEED = 0.6;

    // 手動ローテーション速度のデフォルト値
    const DEFAULT_LOTATION_SPEED = -0.1;

    // ズームの範囲（最小値）のデフォルト値
    const DEFAULT_MIN_DISTANCE = 0;

    // ズームの範囲（最大値）のデフォルト値
    const DEFAULT_MAX_DISTANCE = 5;

    // ズーム速度のデフォルト値
    const DEFAULT_ZOOM_SPEED = 10;

    /**
     * パノラマ画像設置枚数
     *
     * @return int
     */
    public static function imageCount()
    {
        // フィルターフック
        return apply_filters('wpv_image_count', 5);
    }

    /**
     * プラグインのURLのパスを返す
     *
     * @return string
     */
    public static function pluginUrlPath()
    {
        return plugins_url() . '/' . basename(WPV_PLUGIN_PATH) . '/';
    }

    /**
     * パノラマ画像表示に関する設定値を取得
     */
    public static function getSettingValue()
    {
        $setting_value = [];

        // オートローテーション
        $setting_value['autolotation'] = (DAO::getOption('autolotation') === 'false') ? 'false' : 'true';
        // オートローテーション速度
        $setting_value['autolotation_speed'] = (DAO::getOption('autolotation_speed') === false) ? Cmn::DEFAULT_AUTOLOTATION_SPEED : (float)DAO::getOption('autolotation_speed');
        // 手動ローテーション速度
        $setting_value['lotation_speed'] = (DAO::getOption('lotation_speed') === false) ? Cmn::DEFAULT_LOTATION_SPEED : (float)DAO::getOption('lotation_speed');
        // ズームの範囲（最小値）
        $setting_value['min_distance'] = (DAO::getOption('min_distance') === false) ? Cmn::DEFAULT_MIN_DISTANCE : (float)DAO::getOption('min_distance');
        // ズームの範囲（最大値）
        $setting_value['max_distance'] = (DAO::getOption('max_distance') === false) ? Cmn::DEFAULT_MAX_DISTANCE : (float)DAO::getOption('max_distance');
        // ズーム速度
        $setting_value['zoom_speed'] = (DAO::getOption('zoom_speed') === false) ? Cmn::DEFAULT_ZOOM_SPEED : (float)DAO::getOption('zoom_speed');
        // WP Panorama Viewに関するデータの削除
        $setting_value['delete_flg']  = (int)DAO::getOption('delete_flg', 0);
        if ($setting_value['delete_flg'] != 0 && $setting_value['delete_flg'] != 1) {
            $setting_value['delete_flg'] = 0;
        }

        return $setting_value;
    }
}
