(function ($) {
  try {
    new THREE.WebGLRenderer();
  } catch ( e ) {
    $('.panorama-wrap').html('申し訳ございません。<br>お客様のブラウザはパノラマ画像の表示に対応しておりません。<br>別の最新のブラウザにて再度お確かめください。');
    return false;
  }

  var PanoramaView = function(image) {
    this.objectId = PanoramaView.counter++;
    this.image = image;
    this.windowHeight = window.innerHeight;
    this.rect = this.image.getBoundingClientRect();

    this.element;        // パノラマ画像のキャンバス
    this.scene;
    this.camera;
    this.renderer;
    this.controls;
    this.frameId;
    this.geometry;
    this.spheres = [];
    this.initFlag = false;    // セッティング完了フラグ
    this.renderStart = null;  // レンダリングとその時間の制御
    this.initTimer = null;
    this.initStartFlag = false;
    this.sliderSet = null;
  };

  PanoramaView.counter = 0;

  // キャンバスのサイズを取得
  PanoramaView.prototype.getWidthHeight = function() {
    var _ = this;

    var target = _.image.querySelector('.panorama-stage');
    return [target.clientWidth, target.clientHeight];
  };

  // 視点を初期化
  PanoramaView.prototype.resetCameraPosition = function() {
    var _ = this;

    _.camera.position.set(3, 0.15, 0.15);
    _.controls.target.set(0.15, 0, 0);
  };

  PanoramaView.prototype.initView = function() {
    var _ = this;

    // キャンバスのサイズを取得
    var widthHeight = _.getWidthHeight();
    var width = widthHeight[0];
    var height = widthHeight[1];

    // シーンの作成
    _.scene = new THREE.Scene();

    // カメラの作成
    _.camera = new THREE.PerspectiveCamera(60, width / height, 1, 1000);
    _.camera.position.set(3, 0.15, 0.15);
    _.scene.add(_.camera);

    // 球体の形状を作成
    _.geometry = new THREE.SphereGeometry(5, 60, 40);
    _.geometry.scale(-1, 1, 1);

    // 画像データ取得
    var loadFile = _.image.querySelector('.active img').getAttribute('data-src');

    var loader = new THREE.TextureLoader();
    loader.load(loadFile, function(texture) {
      //　マテリアルの作成
      var material = new THREE.MeshBasicMaterial({
        map: texture
      });

      // 球体(形状)にマテリアル(質感)を貼り付けて物体を作成
      var sphere = new THREE.Mesh(_.geometry, material);

      //　シーンに追加
      _.scene.add(sphere);

      var imgNo = _.image.querySelector('.active img').getAttribute('data-no');
      _.spheres[imgNo] = sphere;

      // レンダラーの作成
      _.renderer = new THREE.WebGLRenderer();
      // レンダラーをwindowサイズに合わせる
      _.renderer.setSize(width, height);
      _.renderer.setClearColor(0xDCDCDC, 1.0);
      _.element = _.renderer.domElement;
      _.image.querySelector('.panorama-stage').innerHTML = '';
      _.image.querySelector('.panorama-stage').appendChild(_.element);
      _.renderer.render(_.scene, _.camera);

      // 視点操作
      _.setOrbitControls();

      // ローディングアイコンの非表示
      $(_.image).find('.panorama-loading').fadeOut(500);

      // セッティング完了
      _.initFlag = true;
    });
  };

  // 移動・回転の制御
  PanoramaView.prototype.setOrbitControls = function() {
    var _ = this;

    _.controls = new THREE.OrbitControls(_.camera, _.element);
    _.controls.target.set(0.15, 0, 0);

    // 視点操作のイージングをONにする
    _.controls.enableDamping = true;
    // 視点操作のイージングの値
    _.controls.dampingFactor = 0.2;
    // ローテーションの速さ
    _.controls.rotateSpeed = wpvGl.rotateSpeed;
    // ズーム速度
    _.controls.zoomSpeed = wpvGl.zoomSpeed;
    // ズームの範囲
    _.controls.minDistance = wpvGl.minDistance;
    _.controls.maxDistance = wpvGl.maxDistance;
    // パン操作禁止
    _.controls.enablePan = false;
    // オートローテーション
    if (wpvGl.autolotation === true) {
      _.controls.autoRotate = true;
      // 停止ボタンに切り替える
      _.image.querySelector('.panorama-control').classList.remove('play');
      _.image.querySelector('.panorama-control').classList.add('pause');
    } else if (wpvGl.autolotation === false) {
      _.controls.autoRotate = false;
      // プレイボタンに切り替える
      _.image.querySelector('.panorama-control').classList.remove('pause');
      _.image.querySelector('.panorama-control').classList.add('play');
    }
    _.controls.autoRotateSpeed = wpvGl.autoRotateSpeed;
  };

  // アニメーションの実行
  PanoramaView.prototype.renderLoop = function(timestamp) {
    var _ = this;

    // 実行時間をセット
    if (!_.renderStart) {
      _.renderStart = timestamp;
    }
    // 実行時間の計測
    var progress = timestamp - _.renderStart;

    if (progress < (5 * 1000 * 60)) {
      // 実行時間が5分を過ぎていない場合はアニメーションを継続
      _.frameId = requestAnimationFrame(_.renderLoop.bind(_));

      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }
    } else {
      // 実行時間が5分を過ぎた場合はアニメーションを終了
      cancelAnimationFrame(_.frameId);

      _.controls.autoRotate = false;
      _.image.querySelector('.panorama-control').classList.remove('pause');
      _.image.querySelector('.panorama-control').classList.add('play');
    }

    _.controls.update();
    _.renderer.render(_.scene, _.camera);
  };

  // リサイズ処理
  PanoramaView.prototype.handleResize = function() {
    var _ = this;

    // セッティングが完了していない場合は何もしない
    if (!_.initFlag) {
      return;
    }

    // キャンバスのサイズを取得
    var widthHeight = _.getWidthHeight();
    var width = widthHeight[0];
    var height = widthHeight[1];

    _.renderer.setSize(width, height);
    _.renderer.render(_.scene, _.camera);
    _.camera.aspect = width / height;
    _.camera.updateProjectionMatrix();

    _.handleResizeSlider();
  };

  PanoramaView.prototype.renderUpdate = function() {
    var _ = this;

    var imgNo = _.image.querySelector('.active img').getAttribute('data-no');

    if (typeof _.spheres[imgNo] === 'undefined') {
      // 画像データ取得
      var loadFile = _.image.querySelector('.active img').getAttribute('data-src');
      var loader = new THREE.TextureLoader();
      loader.load(loadFile, function(texture) {

        // 元のパノラマ画像の削除
        for (key in _.spheres) {
          _.scene.remove(_.spheres[key]);
        }

        //　マテリアルの作成
        var material = new THREE.MeshBasicMaterial({
          map: texture
        });

        // 球体(形状)にマテリアル(質感)を貼り付けて物体を作成
        var sphere = new THREE.Mesh(_.geometry, material);
        _.scene.add(sphere);
        _.spheres[imgNo] = sphere;

        // 視点操作初期化
        _.resetCameraPosition();

        $(_.image).find('.panorama-loading').fadeOut(500);

        // セッティング完了
        _.initFlag = true;
      });
    } else {

      // 元のパノラマ画像の削除
      for (key in _.spheres) {
        _.scene.remove(_.spheres[key]);
      }

      _.scene.add(_.spheres[imgNo]);

      // 視点操作初期化
      _.resetCameraPosition();

      $(_.image).find('.panorama-loading').fadeOut(500);

      // セッティング完了
      _.initFlag = true;
    }
  };

  // リサイズイベントを検知してリサイズ処理を実行
  PanoramaView.prototype.addEventListener = function() {
    var _ = this;

    var resizeQueue = null;
    // ブラウザサイズを変更した場合にパノラマ画像要素のリサイズを行う
    window.addEventListener('resize', function() {
      clearTimeout(resizeQueue);
      resizeQueue = setTimeout(_.handleResize.bind(_), 100);
    });

    // パノラマ画像のレンダリングを開始する
    window.addEventListener('load', _.initRender.bind(_), false);
    window.addEventListener('scroll', _.initRender.bind(_), false);

    // ウィンドウからフォーカスが外れた場合にアニメーションをストップする
    window.addEventListener('blur', function() {
      if (typeof _.frameId !== 'undefined') {
        cancelAnimationFrame(_.frameId);
      }
    });
    // ウィンドウにフォーカスが当たった場合にアニメーションを再開する
    window.addEventListener('focus', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      _.renderStart = null;
      cancelAnimationFrame(_.frameId);
      _.frameId = requestAnimationFrame(_.renderLoop.bind(_));
    });
  };

  // パノラマビューアーにフォーカスされた場合に、描画時間をリセットして延長する
  PanoramaView.prototype.initClickExtension = function() {
    var _ = this;

    $(_.image).on('mousedown touchstart', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      _.renderStart = null;
      cancelAnimationFrame(_.frameId);
      _.frameId = requestAnimationFrame(_.renderLoop.bind(_));
    });
  };

  PanoramaView.prototype.initThumbnailClick = function() {
    var _ = this;

    $(_.image).find('ul li').on('click', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }
      _.initFlag = false;

      $(_.image).find('.panorama-loading').show();

      $(this).addClass('active');
      $(this).siblings().removeClass('active');

      if (wpvGl.autolotation === true) {
        _.controls.autoRotate = true;
        _.image.querySelector('.panorama-control').classList.remove('play');
        _.image.querySelector('.panorama-control').classList.add('pause');
      } else if (wpvGl.autolotation === false) {
        _.controls.autoRotate = false;
        _.image.querySelector('.panorama-control').classList.remove('pause');
        _.image.querySelector('.panorama-control').classList.add('play');
      }

      _.renderUpdate();
    });
  };

  PanoramaView.prototype.initControlClick = function() {
    var _ = this;

    $(_.image).find('.panorama-control').on('click', function() {
      // セッティングが完了していない場合は何もしない
      if (!_.initFlag) {
        return;
      }

      if (this.classList.contains('play')) {
        _.controls.autoRotate = true;
        $(this).removeClass('play').addClass('pause');
      } else {
        _.controls.autoRotate = false;
        $(this).removeClass('pause').addClass('play');
      }
    });
  };

  // 画像データを取得する
  PanoramaView.prototype.loadImage = function() {
    var _ = this;

    var imageList = _.image.getElementsByClassName('wpv-lazyload');
    var images = Array.prototype.slice.call(imageList, 0);
    images.forEach(function(image) {
      var imagePath = image.getAttribute('data-thumbsrc');
      image.setAttribute('src', imagePath);
    });
  };

  // パノラマ画像のレンダリングをスタート
  PanoramaView.prototype.initRender = function() {
    var _ = this;

    if (_.initStartFlag) {
      return;
    }

    _.rect = this.image.getBoundingClientRect();
    clearTimeout(_.initTimer);
    _.initTimer = setTimeout(function() {
      if (_.windowHeight > _.rect.top) {
        _.initStartFlag = true;
        _.loadImage();
        _.initView();
        _.frameId = _.renderLoop(0);
        _.sliderSet = new Slider(_.image);
        if (_.sliderSet.isOverflow($('.slider'))) {
          _.sliderSet.slideBtnToggle();
        }
      }
    }, 100);
  };

  // ブラウザリサイズ時のサムネイルのスライドボタンの表示・非表示
  PanoramaView.prototype.handleResizeSlider = function() {
    var _ = this;

    if (!_.sliderSet) {
      return;
    }

    var isOver = _.sliderSet.isOverflow($('.slider'));
    var change = (isOver == _.sliderSet.overflowSwitch);
    if (change && isOver) {
      if (_.sliderSet.slideCurrent !== 0) {
        _.sliderSet.slideCurrent = 0;
        _.sliderSet.sliding();
      }
      _.sliderSet.slideBtnToggle();
      _.sliderSet.overflowSwitch = false;
    } else if (change && !isOver) {
      if (_.sliderSet.slideCurrent !== 0) {
        _.sliderSet.slideCurrent = 0;
        _.sliderSet.sliding();
      }
      $(_.image).find('.slider .slider-prev').hide();
      $(_.image).find('.slider .slider-next').hide();
      _.sliderSet.overflowSwitch = true;
    }
    if (_.sliderSet.fitWidth($('.slider'))) {
      $(_.image).find('.slider .slider-next').hide();
    } else {
      $(_.image).find('slider .slider-next').show();
    }
  };

  // サムネイルクリック時にスライドさせる
  PanoramaView.prototype.thumbnailSliding = function() {
    var _ = this;

    $(_.image).find('.slider .slider-prev').on('click', function() {
      if (!_.sliderSet) {
        return;
      }

      _.sliderSet.slideCurrent--;
      _.sliderSet.sliding();
    });

    $(_.image).find('.slider .slider-next').on('click', function() {
      if (!_.sliderSet) {
        return;
      }

      _.sliderSet.slideCurrent++;
      _.sliderSet.sliding();
    });
  };

  // パノラマ画像の初期設定
  PanoramaView.prototype.init = function() {
    var _ = this;

    _.addEventListener();
    _.initClickExtension();
    _.initThumbnailClick();
    _.initControlClick();
    _.thumbnailSliding();
  };

  // パノラマ画像のサムネイルの設定
  var Slider = function(image) {
    this.image = image;
    this.overflowSwitch = true;
    this.padding = 10;
    this.slideWidth = $(this.image).find('.slide').outerWidth();
    this.slideNum = $(this.image).find('.slide').length;
    this.slideCurrent = 0;
    this.overflowWidth = (this.slideWidth + this.padding) * this.slideNum;
  };

  // パノラマ画像のラッパー要素の幅とサムネイルの合計の幅を比較
  Slider.prototype.isOverflow = function(target) {
    var _ = this;

    return (target.outerWidth() < _.overflowWidth);
  };

  // パノラマ画像のラッパー要素の幅と表示中のサムネイルの幅を比較
  Slider.prototype.fitWidth = function(target) {
    var _ = this;

    var tmpWidth = (_.slideWidth + _.padding) * (_.slideNum - _.slideCurrent);
    return (target.outerWidth() > tmpWidth);
  };

  // パノラマ画像のサムネイルのスライドボタンの表示・非表示
  Slider.prototype.slideBtnToggle = function() {
    var _ = this;

    if (_.slideCurrent === 0) {
      $(_.image).find('.slider .slider-prev').hide();
    }
    if (_.fitWidth($('.slider'))) {
      $(_.image).find('.slider .slider-next').hide();
    } else {
      $(_.image).find('.slider .slider-next').show();
    }
    if (_.slideCurrent > 0) {
      $(_.image).find('.slider .slider-prev').show();
    }
  };

  // パノラマ画像のサムネイルをスライドさせる
  Slider.prototype.sliding = function() {
    var _ = this;

    $(_.image).find('.slideSet').stop().animate({
      left: _.slideCurrent * -(_.slideWidth + _.padding)
    });

    _.slideBtnToggle();
  };

  // パノラマ画像を表示ラッパー要素を取得
  var imageList = document.querySelectorAll('.panorama-wrap');
  // オブジェクト→配列
  var images = Array.prototype.slice.call(imageList, 0);
  // パノラマ画像をそれぞれ表示
  images.forEach(function(image) {
    var panorama = new PanoramaView(image);
    panorama.init();
  });
})(jQuery);
